package com.ruoyi.common.utils.collection;

import com.ruoyi.common.utils.DateUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author wjh
 * 2024/4/29
 */
public class CollectionUtils extends org.springframework.util.CollectionUtils {

    /**
     * 是否为空，包括空集合、空元素
     * @param list
     * @return
     */
    public static boolean isEmptyElement(List<?> list) {
        if (list == null) {
            return true;
        }
        // 过滤空元素
        List<?> collect = list.stream().filter(Objects::nonNull).collect(Collectors.toList());
        return org.springframework.util.CollectionUtils.isEmpty(collect);
    }

    public static boolean isNotEmptyElement(List<?> list) {
        return !isEmptyElement(list);
    }

    /**
     * 填充列表中的元素
     * @param oldList 旧列表
     * @param startDate 起始日期
     * @param endDate 结束日期
     * @param clazz 填充元素的类
     */
    public static List<? super FillAble> fillElement(List<? extends FillAble> oldList, Date startDate, Date endDate,Class<? extends FillAble> clazz ) throws InstantiationException, IllegalAccessException {
        List<FillAble> newList = new ArrayList<>();
        while (startDate.compareTo(endDate) <= 0) {
            Date finalI = startDate;
            FillAble fillable = oldList.stream().filter(item -> DateUtils.getYYYY_MM_DD(item.fillDate()).equals(DateUtils.getYYYY_MM_DD(finalI))).findFirst().orElse(null);
            if (fillable == null) {
                fillable = clazz.newInstance().fillElement(finalI);
            }
            newList.add(fillable);
            startDate = DateUtils.addDays(startDate,1);
        }
        return newList;
    }


    // TODO 未经验证
    /**
     * 补全列表中的日期数据，对于缺失的日期使用提供的工厂方法生成对象。
     *
     * @param dataList        原始数据列表，元素需有getDateMethod返回Date类型属性的方法
     * @param dateGetter      获取元素日期属性的方法引用
     * @param customFactory   当日期缺失时，用于生成对应元素的对象工厂方法
     * @param <T>             数据列表中元素的类型
     * @return 完整日期序列下的数据列表，缺失的日期已由customFactory填充
     */
    public static <T> List<T> completeMissingDates(List<T> dataList,
                                                   Function<T, Date> dateGetter,
                                                   Function<Date, T> customFactory) {
        if (dataList == null || dataList.isEmpty()) {
            return Collections.emptyList();
        }

        List<Date> dates = dataList.stream()
                .map(dateGetter)
                .sorted()
                .collect(Collectors.toList());

        Date startDate = dates.get(0);
        Date endDate = dates.get(dates.size() - 1);

        List<Date> fullDates = new ArrayList<>();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(startDate);

        while (!calendar.getTime().after(endDate)) {
            fullDates.add(calendar.getTime());
            calendar.add(Calendar.DATE, 1);
        }

        Map<Date, T> dateToObjectMap = dataList.stream()
                .collect(Collectors.toMap(dateGetter, Function.identity()));

        return fullDates.stream()
                .map(date -> dateToObjectMap.getOrDefault(date, customFactory.apply(date)))
                .collect(Collectors.toList());
    }

    public static boolean isNotEmpty(List<?> deviceList) {
        return !isEmpty(deviceList);
    }

    /**
     * 集合映射，去除空值、重复值
     */
    public static <T, R> List<R> map(List<T> list, Function<? super T, ? extends R> keyMapper) {
        return list.stream().map(keyMapper).filter(Objects::nonNull).distinct().collect(Collectors.toList());
    }
}
