package com.ruoyi.ss.account.mapper;

import com.ruoyi.ss.account.domain.SmAccount;
import com.ruoyi.ss.account.domain.SmAccountQuery;
import com.ruoyi.ss.account.domain.SmAccountVo;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 用户账户Mapper接口
 *
 * @author ruoyi
 * @date 2024-03-26
 */
public interface SmAccountMapper
{
    /**
     * 查询用户账户
     *
     * @param accountId 用户账户主键
     * @return 用户账户
     */
    public SmAccountVo selectSmAccountByAccountId(Long accountId);

    /**
     * 查询用户账户列表
     *
     * @param smAccount 用户账户
     * @return 用户账户集合
     */
    public List<SmAccountVo> selectSmAccountList(SmAccountQuery smAccount);

    /**
     * 新增用户账户
     *
     * @param smAccount 用户账户
     * @return 结果
     */
    public int insertSmAccount(SmAccount smAccount);

    /**
     * 修改用户账户
     *
     * @param smAccount 用户账户
     * @return 结果
     */
    public int updateSmAccount(SmAccount smAccount);

    /**
     * 删除用户账户
     *
     * @param accountId 用户账户主键
     * @return 结果
     */
    public int deleteSmAccountByAccountId(Long accountId);

    /**
     * 批量删除用户账户
     *
     * @param accountIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteSmAccountByAccountIds(Long[] accountIds);

    /**
     * 根据账号获取数据
     * @param accountNo 账号
     * @param accountType 类型
     */
    SmAccountVo selectByAccountNo(@Param("accountNo") String accountNo, @Param("accountType") String accountType);

    /**
     * 为用户设置默认的账户
     * @param userId 用户id
     * @param accountId 账户id
     */
    int setDefault(@Param("userId") Long userId, @Param("accountId") Long accountId);
}
