package com.ruoyi.ss.refund.service.impl;

import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.SnowFlakeUtil;
import com.ruoyi.ss.refund.domain.Refund;
import com.ruoyi.ss.refund.domain.enums.RefundStatus;
import com.ruoyi.ss.refund.service.RefundConverter;
import com.ruoyi.ss.transactionBill.domain.TransactionBill;
import com.ruoyi.ss.transactionBill.domain.TransactionBillVo;
import com.ruoyi.ss.transactionBill.domain.dto.BillRefundDTO;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;

/**
 * @author wjh
 * 2024/7/9
 */
@Service
public class RefundConverterImpl implements RefundConverter {

    @Autowired
    private TransactionBillService transactionBillService;

    @Override
    public Refund toPo(BillRefundDTO dto) {
        if (dto == null) {
            return null;
        }
        TransactionBillVo bill = transactionBillService.selectSmTransactionBillByBillId(dto.getBillId());
        ServiceUtil.assertion(bill == null, "待退款订单不存在");

        // 按比例计算退款金额
        BigDecimal refundAmount = dto.getRefundAmount();
        BigDecimal refundRate = refundAmount.divide(bill.getMoney(), 2, RoundingMode.HALF_UP); // 退款金额比例
        BigDecimal refundServiceAmount = bill.getServiceCharge().multiply(refundRate); // 退款的手续费
        BigDecimal refundMchAmount = refundAmount.subtract(refundServiceAmount);    // 退款的商户余额

        Refund refund = new Refund();
        refund.setBillId(dto.getBillId());
        refund.setAmount(dto.getRefundAmount());
        refund.setStatus(RefundStatus.REFUNDING.getStatus());
        refund.setReason(String.format("充值订单%s退款", bill.getBillNo()));
        refund.setMchAmount(refundMchAmount);
        refund.setServiceAmount(refundServiceAmount);
        refund.setUserType(dto.getUserType());
        refund.setUserName(dto.getUserName());
        refund.setUserId(dto.getUserId());
        return refund;
    }
}
