package com.ruoyi.web.controller.app;

import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.annotation.Anonymous;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.pay.wx.domain.NotifyEventType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.http.HttpUtils;
import com.ruoyi.ss.payBill.service.PayBillService;
import com.ruoyi.ss.refund.service.RefundService;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.ss.wxPay.domain.enums.AttachEnums;
import com.ruoyi.ss.wxPay.service.IWxPayService;
import com.wechat.pay.java.core.exception.ValidationException;
import com.wechat.pay.java.core.notification.Notification;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.refund.model.RefundNotification;
import com.wechat.pay.java.service.refund.model.Status;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;

/**
 * @author 辉
 * 2024/3/11
 */
@Api(tags = "支付")
@RestController
@RequestMapping("/app/pay")
@Slf4j
public class AppPayController {

    @Autowired
    private IWxPayService wxPayService;

    @Autowired
    private TransactionBillService transactionBillService;

    @Autowired
    private RefundService refundService;

    @Autowired
    private PayBillService payBillService;

    @ApiOperation("微信支付")
    @GetMapping("/wx/{billNo}")
    public AjaxResult wxPay(@PathVariable @ApiParam("订单编号") String billNo) {
        return AjaxResult.success(wxPayService.prepayWithRequestPayment(billNo));
    }

    // 微信支付结果通知
    @ApiOperation("微信支付通知")
    @PostMapping("/notify/wx")
    @Anonymous
    public ResponseEntity<Boolean> wxPayNotify(HttpServletRequest request) {
        try {
            log.info("收到微信支付回调{}", request);

            // 转为微信支付业务结果对象
            Transaction transaction = wxPayService.toTransaction(request);
            // 支付成功
            if (wxPayService.isSuccess(transaction)) {
                // 根据附加信息，判断是哪个订单
                String attach = transaction.getAttach();
                if (AttachEnums.TRANSACTION_BILL.getAttach().equals(attach)) {
                    // 旧版充值订单
                    transactionBillService.rechargeSuccess(transaction.getOutTradeNo(), new Date(transaction.getSuccessTime()));
                } else if (AttachEnums.PAY_BILL.getAttach().equals(attach)){
                    // 新版支付订单
                    payBillService.handleSuccess(transaction.getOutTradeNo(), DateUtils.toLocalDateTimeByISO(transaction.getSuccessTime()));
                }
            }
        } catch (ValidationException e) {
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body(null);
        } catch (Exception e) {
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(null);
        }
        return ResponseEntity.status(HttpStatus.OK).body(null);
    }

    // 微信退款回调
    @PostMapping("/notify/wx/refund")
    @Anonymous
    public ResponseEntity<Boolean> wxPayNotifyRefund(HttpServletRequest request) {
        try {
            String body = HttpUtils.getBody(request);
            log.info("【微信退款回调】接收对象 : " + JSON.toJSONString(body));
            // 解析通知数据
            Notification notification = JSON.parseObject(body, Notification.class);
            log.info("【微信退款回调】转换成notification: " + JSON.toJSONString(notification));
            // 退款成功通知
            if (NotifyEventType.REFUND_SUCCESS.getValue().equals(notification.getEventType())) {
                // 验签、解密并转换成 RefundNotification
                RefundNotification refundNotification = wxPayService.checkAndParse(request, body, RefundNotification.class);
                log.info("【微信退款回调】转换成RefundNotification: " + JSON.toJSONString(refundNotification));
                if (Status.SUCCESS.equals(refundNotification.getRefundStatus())) {
                    // 退款成功操作
                    String outRefundNo = refundNotification.getOutRefundNo();
                    refundService.handleRefundSuccess(outRefundNo);
                }
            }
        } catch (ValidationException e) {
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body(null);
        } catch (Exception e) {
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(null);
        }
        return ResponseEntity.status(HttpStatus.OK).body(null);
    }

    @ApiOperation("查询支付结果")
    @GetMapping("/result/{billNo}")
    public AjaxResult payResult(@PathVariable @ApiParam("订单编号") String billNo) {
        return AjaxResult.success(transactionBillService.getPayResult(billNo));
    }

    @ApiOperation("微信商户支付通知")
    @PostMapping("/notify/wx/transfer")
    public ResponseEntity<Boolean> wxTransferNotify(HttpServletRequest request) {
        try {
            wxPayService.wxTransferNotify(request);
        } catch (ValidationException e) {
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body(null);
        } catch (Exception e) {
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(null);
        }
        return ResponseEntity.status(HttpStatus.OK).body(null);
    }

}
