package com.ruoyi.web.controller.ss;

import java.util.Arrays;
import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.ruoyi.common.core.domain.ValidGroup;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.suit.domain.Suit;
import com.ruoyi.ss.suit.domain.SuitBO;
import com.ruoyi.ss.suit.domain.SuitQuery;
import com.ruoyi.ss.suit.domain.SuitVo;
import com.ruoyi.ss.suit.service.SuitValidator;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.ss.suit.service.SuitService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 套餐Controller
 *
 * @author ruoyi
 * @date 2024-04-25
 */
@RestController
@RequestMapping("/ss/suit")
public class SuitController extends BaseController
{
    @Autowired
    private SuitService suitService;

    @Autowired
    private SuitValidator suitValidator;

    @Autowired
    private DeviceService deviceService;

    /**
     * 查询套餐列表
     */
    @PreAuthorize("@ss.hasPermi('ss:suit:list')")
    @GetMapping("/list")
    public TableDataInfo list(SuitQuery suit)
    {
        startPage();
        List<SuitVo> list = suitService.selectSuitList(suit);
        return getDataTable(list);
    }

    /**
     * 导出套餐列表
     */
    @PreAuthorize("@ss.hasPermi('ss:suit:export')")
    @Log(title = "套餐", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, SuitQuery suit)
    {
        List<SuitVo> list = suitService.selectSuitList(suit);
        ExcelUtil<SuitVo> util = new ExcelUtil<SuitVo>(SuitVo.class);
        util.exportExcel(response, list, "套餐数据");
    }

    /**
     * 获取套餐详细信息
     */
    @PreAuthorize("@ss.hasPermi('ss:suit:query')")
    @GetMapping(value = "/{suitId}")
    public AjaxResult getInfo(@PathVariable("suitId") Long suitId)
    {
        return success(suitService.selectSuitBySuitId(suitId));
    }

    /**
     * 新增套餐
     */
    @PreAuthorize("@ss.hasPermi('ss:suit:add')")
    @Log(title = "套餐", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody @Validated(ValidGroup.Create.class) SuitBO suit)
    {
        DeviceVO device = deviceService.selectSmDeviceByDeviceId(suit.getDeviceId());
        if (device == null) {
            return error("设备不存在");
        }
        suit.setUserId(device.getUserId());
        return toAjax(suitService.insertSuit(suit));
    }

    /**
     * 修改套餐
     */
    @PreAuthorize("@ss.hasPermi('ss:suit:edit')")
    @Log(title = "套餐", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody @Validated(ValidGroup.Update.class) SuitBO suit)
    {
        return toAjax(suitService.updateSuit(suit));
    }

    /**
     * 逻辑删除套餐
     */
    @PreAuthorize("@ss.hasPermi('ss:suit:remove')")
    @Log(title = "套餐", businessType = BusinessType.LOGIC_DELETE)
	@DeleteMapping("/{suitIds}")
    public AjaxResult remove(@PathVariable Long[] suitIds)
    {
        List<Long> ids = Arrays.asList(suitIds);
        ServiceUtil.assertion(suitValidator.preLogicDel(ids));
        return toAjax(suitService.logicDel(ids) >= 1);
    }
}
